@@ -145,6 +145,10 @@ def windowfunc(strfunc):
         args = L(*args).fmap(lambda x: getattr(q0.columns, x) if type(x) is str else x)
         return q0.aggregate(lambda x: exprfunc(*args, **kwargs))
     setattr(Query, strfunc.__name__[:-1], qfunc)
+    @wraps(strfunc)
+    def colfunc(col, *args, **kwargs):
+        return qfunc(col.asQuery(), col, *args, **kwargs)
+    setattr(Columns, strfunc.__name__, colfunc)
     return exprfunc
 
 
