@@ -127,6 +127,9 @@ class Query(Table, Monad):
     def isagg(self):
         return (self.columns.values() + self.joincond).any(lambda x: x.isagg())
     
+    def aggedTables(self):
+        return self.allDescendants().filter(lambda x: isinstance(x, AggExpr)).getTables()
+    
     def iswindow(self):
         return self.columns.values().any(lambda x: x.iswindow())
     
@@ -503,8 +506,9 @@ def identifyTable(cond0, cond1, tab0, tab1, res, other):
     # tab1 and tab0 are both joined to the same table by the same thing
     # the extras are just where conds
     
-    # cond0 = res.joincond._filter(tab0, other.joincond.getTables())
-    # cond1 = other.joincond._filter(tab1, res.joincond.getTables())
+    # jtables = res.joincond.getTables() & other.joincond.getTables()
+    # cond0 = res.joincond._filter(tab0, jtables)
+    # cond1 = other.joincond._filter(tab1, jtables)
     
     andcond = cond0 & cond1
     
