@@ -302,11 +302,14 @@ def rerouteAll(table, inner, outer):
 def canRerouteTable(basetable, inner, outer):
     
     # all baseexprs of outer containing basetable
-    allexprs = outer.allExprs().bind(lambda x: L(x) + x.descendants())\
+    allexprs = outer.allDescendants()\
         .filter(lambda x: type(x) is BaseExpr and basetable in x.getTables())
     
     isgrouped = allexprs.fmap(lambda x: isGrouped(x, inner)).all()
     
+    # if not isgrouped:
+    # print('not grouped:', basetable)
+    
     # if the second condition is true then we just add basetable to the groupbys of inner
     return isgrouped or basetable in outer.groupbys.filter(lambda x: x.isPrimary()).getTables()
 
@@ -438,6 +441,9 @@ def canMerge(self, subquery=False):
     else:
         subq = True
     
+    if self.subQueries().bind(lambda x: x.aggedTables()).len() > 1:
+        return False
+    
     return canmerge and self.joincond.children.filter(lambda x: isInvalid(x, isjc=True)).notExists() \
         and subq and (self.columns.values() + self.groupbys).filter(lambda x: isInvalid(x, isjc=False)).notExists()
 
@@ -449,9 +455,15 @@ def canMergeTable(tab0, tab1, subquery=False):
         return False
         # return tab1.joincond == tab0.joincond
     
-    if tab1 in tab0.getTables() and not canMerge(tab1):
+    if (tab1 in tab0.getTables() and not canMerge(tab1)) or (tab0 in tab1.getTables() and not canMerge(tab0)):
         return False
     
+    
+    if tab1 not in tab0.getTables() and tab0 not in tab1.getTables():
+        tab11 = copy(tab1)
+        tab00 = copy(tab0)
+        return (tab00 @ tab11).aggedTables() == tab00.aggedTables()
+    
     go = (tab0.groupbys <= tab1.groupbys and tab0 in tab1.getTables() and not isSum(tab0) and not isSum(tab1)) or \
               (tab1.groupbys <= tab0.groupbys and tab1 in tab0.getTables() and not isSum(tab0) and not isSum(tab1)) or \
               ((isSum(tab0) or isSum(tab1)) and tab0.joincond == tab1.joincond and tab0.groupbys == tab1.groupbys) or \
@@ -468,26 +480,33 @@ def canMergeTable(tab0, tab1, subquery=False):
 def mergeGrouped(self, debug=False, subquery=False):
     # merge tables that have the same groupbys, because only then does it make sense to merge them
     
-    checktables = self.getTables()
-    
-    alltablegroups = L()
+    # checktables = self.subQueries()
     
     if canMerge(self, subquery=subquery):
-        alltablegroups = L(L(self) + checktables.filter(lambda x: x.groupbys.exists() #and not hasExists(x, self)
-                                                          and x.groupbys <= self.groupbys #.fmap(lambda x: x.getRef())
-                                                          and not x.allDescendants().filter(lambda y: type(y) is AggExpr and y.func.__name__ == 'sum_')
-                                                          and not x.limitvar
-                                                            )).filter(lambda x: len(x) > 1)
+        checktables = L(self) + self.subQueries()
+    else:
+        checktables = self.subQueries()
     
-    if not alltablegroups:
-        alltablegroups = checktables.filter(lambda x: x.isQuery()
-                                            and not x.allDescendants().filter(lambda y: type(y) is AggExpr and y.func.__name__ == 'sum_')
-                                            )\
-                           .groupby(lambda x: (x.groupbys, x.leftjoin))\
-                           .filter(lambda x: len(x.value) > 1)\
-                           .fmap(lambda x: x.value)
+    # alltablegroups = L()
+    
+    alltablegroups = checktables.partition(lambda x, y: canMergeTable(x, y, subquery=subquery)).filter(lambda x: len(x) > 1)
+    # if canMerge(self, subquery=subquery):
+    # else:
+        # alltablegroups = self.subQueries().partition(lambda x, y: canMergeTable(x, y, subquery=subquery)).filter(lambda x: len(x) > 1)
+    #     alltablegroups = L(L(self) + checktables.filter(lambda x: x.groupbys.exists() #and not hasExists(x, self)
+    #                                                       and x.groupbys <= self.groupbys #.fmap(lambda x: x.getRef())
+    #                                                       and not x.allDescendants().filter(lambda y: type(y) is AggExpr and y.func.__name__ == 'sum_')
+    #                                                       and not x.limitvar
+    #                                                         )).filter(lambda x: len(x) > 1)
+    
+    # if not alltablegroups:
+    #     alltablegroups = checktables.filter(lambda x: x.isQuery()
+    #                                         and not x.allDescendants().filter(lambda y: type(y) is AggExpr and y.func.__name__ == 'sum_')
+    #                                         )\
+    #                        .groupby(lambda x: (x.groupbys, x.leftjoin))\
+    #                        .filter(lambda x: len(x.value) > 1)\
+    #                        .fmap(lambda x: x.value)
     
-    alltablegroups = (L(self) + self.subQueries()).partition(lambda x, y: canMergeTable(x, y, subquery=subquery)).filter(lambda x: len(x) > 1)
     
     
     for tablegroup in alltablegroups:
@@ -650,15 +669,15 @@ def reduceQuery(self, debug=False, subquery=False):
         moveOutAll(inner=newtable, outer=self, debug=debug)
     
     
-    # STEP 4: COPY TABLES THAT CAN'T BE REROUTED
-    for table in getRerouteTables(self):
-        newtable = (table.parents - self)[0]
-        if canRerouteTable(table, inner=newtable, outer=self): continue
-        copyJoinconds(table, newtable, self)
-        copyTable(table, self, addcond=False)
-        if debug:
-            print("COPY TABLE", table)
-            print(self.sql(reduce=False, debug=True))
+    # # STEP 4: COPY TABLES THAT CAN'T BE REROUTED
+    # for table in getRerouteTables(self):
+    #     newtable = (table.parents - self)[0]
+    #     if canRerouteTable(table, inner=newtable, outer=self): continue
+    #     # copyJoinconds(table, newtable, self)
+    #     copyTable(table, self, addcond=True)
+    #     if debug:
+    #         print("COPY TABLE", table)
+    #         print(self.sql(reduce=False, debug=True))
     
     
     # STEP 5: ACTUALLY REROUTE
